﻿namespace TabsStudioMvcGroup
{
    public class MvcGroup : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IStyler
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;
            engine.TabCreated += OnTabCreated;
        }

        public void OnDisconnection()
        {
            engine.TabCreated -= OnTabCreated;
        }

        public string GetNamespacesForResourceDictionary()
        {
            return "xmlns:TabsStudioMvcGroup=\"clr-namespace:TabsStudioMvcGroup;assembly=MvcGroup\"";
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            TabInfo info = new TabInfo(e.Tab.TabExtensions[0].Window);
            Properties.SetTabInfo(e.Tab.TabItem, info);
            Properties.SetGroupName(e.Tab.TabItem, info.groupName);
            Properties.SetIsController(e.Tab.TabItem, info.type == TabType.Controller);
            Properties.SetIsView(e.Tab.TabItem, info.type == TabType.View);
            if (info.type != TabType.Unknown)
            {
                System.Windows.UIElement newTabItemPosition = FindNewTabItemPosition(info, e.Tabs, e.Tab);
                if (newTabItemPosition != null)
                {
                    e.Tabs.TabPanel2.Children.Remove(e.Tab.TabItem);
                    e.Tabs.TabPanel2.Children.Insert(e.Tabs.TabPanel2.Children.IndexOf(newTabItemPosition), 
                        e.Tab.TabItem);
                }
            }
        }

        private System.Windows.UIElement FindNewTabItemPosition(TabInfo info, 
            TabsStudioExt.Tabs tabs, TabsStudioExt.Tab tab)
        {
            bool ownGroupFound = false;
            foreach (System.Windows.UIElement i in tabs.TabPanel2.Children)
            {
                if (i == tab.TabItem)
                    continue;

                TabInfo currentInfo = Properties.GetTabInfo(i);
                if (currentInfo.type != TabType.Unknown)
                {
                    if (System.String.Compare(info.groupName, currentInfo.groupName, true) == 0 &&
                        System.String.Compare(info.rootDirectory, currentInfo.rootDirectory, true) == 0)
                    {
                        if (info.type == TabType.Controller)
                            return i;
                        ownGroupFound = true;
                        continue;
                    }
                }
                if (ownGroupFound)
                    return i;
            }
            return null;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
    }
}
